/*
 * Copyright (c) 2021 Macrometa Corp All rights reserved.
 */

use proc_macro::TokenStream;
use quote::quote;
use syn::parse_macro_input;

pub fn expand_macro(item: TokenStream) -> TokenStream {
    let function_definition = parse_macro_input!(item as syn::ItemFn);
    let function_name = &function_definition.sig.ident;

    let function_main = quote! {

        use std::io::stdin;

        fn main() {
            match execute() {
                Ok(res) => println!("{}", res),
                Err(err) => println!("{}", get_error_response(&err)),
            }
        }

        fn execute() -> Result<String, String> {
            use photoniq_faas_sdk::{
                binding::http::{ HttpRequest, HttpResponse}
            };

            let http_request: HttpRequest =
                HttpRequest::new_from_stdin(stdin()).map_err(|_| "http request parsing".to_string())?;
            let http_response: HttpResponse =
                #function_name(http_request).map_err(|_| "function execution".to_string())?;
            http_response.to_string().map_err(|_| "http response parsing".to_string())
        }

        fn get_error_response(cause: &str) -> String {
            use photoniq_faas_sdk::{
                binding::http::{ HttpStatus, HttpResponseBuilder}
            };

            let msg = format!("Error running the function on {}.", cause);
            let res = HttpResponseBuilder::new(HttpStatus::InternalServerError)
                .body(msg.clone().into())
                .build();
            res.to_string().unwrap()
        }

        #function_definition
    };

    TokenStream::from(function_main)
}
